c gam3g.f - Gateway function for gam3.f 
c 
c Arguments
c
c nrhs: number of right-hand side arguments
c nlhs: number of left-hand side arguments
c prhs: length nrhs array of pointers to the right-hand side matrices
c plhs: length nlhs array of pointers to the left-hand side matrices
c
c !fmex gam3.f gam3g.f to create Mex-File gam3.mex
c GAM3.MEX: Variogram of regularly Spaced 3-D Data
c Equivalent to GAM3m.f without the check subroutine 
c
c Caroline Lafleur
c 30/07/96
c      

      subroutine mexFunction(nlhs, plhs, nrhs, prhs)
      include 'gam3.inc'
c
c Input to the MEX-file
c
      implicit none
      integer plhs(*) , prhs(*)
      integer nlhs    , nrhs
c
      integer mxCreateFull, mxGetPr , mxCreateString
      integer mxGetM, mxGetN, mxGetString, mxSetM, mxSetN 
      real    mxGetScalar

c
c keep the above subroutine, argument, and function declarations for use
c in all your fortran mex files.
c---------------------------------------------------------------------
c

c
c GAM3 function arguments in the syntax of the matlab language
c [np,gam,hm,tm,hv,tv] = gam3 (nlag,nx,ny,nz,ndir,ixd,iyd,izd,vr,tmin,tmax,nvarg,ivtail,
c     			          ivhead,ivtype,nvar)
c

c
c Pointers declaration
c
      integer*4 nlag,nx,ny,nz,ndir,ixd,iyd,izd,vr,tmin,tmax,
     +          nvarg,ivtail,ivhead,ivtype,nvar
      integer*4 np,gam,hm,tm,hv,tv

c
c Local variables
c
      integer*4    n,m,nvario,i,j
      integer*4    valeur1,valeur2,valeur3,valeur4,valeur5,
     +             valeur12,valeur16

c
c Check for proper number of arguments
c
      if (nrhs .ne. 16) then 
	call mexErrMsgTxt ('gam3 requires 16 input arguments')
      else if (nlhs .ne. 6) then
	call mexErrMsgTxt ('gam3 requires 6 output arguments')
      endif

c
c Some values needed
c
      valeur1 = mxgetscalar (prhs(1))
      valeur2 = mxgetscalar (prhs(2))
      valeur3 = mxgetscalar (prhs(3))
      valeur4 = mxgetscalar (prhs(4))
      valeur5 = mxgetscalar (prhs(5))
      valeur12= mxgetscalar (prhs(12))
      valeur16= mxgetscalar (prhs(16))

c
c Perform some quick error checking:
c
      if(valeur2.lt.1)         stop 'nx is too small: check parameters'
      if(valeur2.gt.MAXX)      stop 'nx is too big: check gam3.inc'
      if(valeur3.lt.1)         stop 'ny is too small: check parameters'
      if(valeur3.gt.MAXY)      stop 'ny is too big: check gam3.inc'
      if(valeur4.lt.1)         stop 'nz is too small: check parameters'
      if(valeur4.gt.MAXZ)      stop 'nz is too big: check gam3.inc'
      if(valeur5.lt.1)       stop 'ndir is too small: check parameters'
      if(valeur5.gt.MAXDIR)  stop 'ndir is too big: check gam3.inc'
      if(valeur16.lt.1)       stop 'nvar is too small: check parameters'
      if(valeur16.gt.MAXVAR)  stop 'nvar is too big: check gam3.inc'
      if(valeur1.lt.1)       stop 'nlag is too small: check parameters'
      if(valeur1.gt.MAXLAG)  stop 'nlag is too big: check gam3.inc'
      if(valeur12.lt.1)      stop 'nvarg is too small: check parameters'
      if(valeur12.gt.MXVARG) stop 'nvarg is too big: check gam3.inc'

c
c Check the dimension of rigth-side matrices
c
c nlag,nx,ny,nz,ndir,nvarg,nvar,tmin,tmax are 1x1 matrices
c ixd,iyd,izd are 1xndir or ndirx1 matrices
c ivtail,ivhead,ivtype are 1xnvarg or nvargx1 matrices
c vr is a vector of length <= maxdat
c
      m = mxGetM (prhs(1))
      n = mxGetN (prhs(1))
      if (m .ne. 1 .or. n .ne. 1) then
	call mexErrMsgTxt ('nalg must be a vector of length 1')
      endif
c
      m = mxGetM (prhs(2))
      n = mxGetN (prhs(2))
      if (m .ne. 1 .or. n .ne. 1) then
	call mexErrMsgTxt ('nx must be a vector of length 1')
      endif
c
      m = mxGetM (prhs(3))
      n = mxGetN (prhs(3))
      if (m .ne. 1 .or. n .ne. 1) then
	call mexErrMsgTxt ('ny must be a vector of length 1')
      endif
c
      m = mxGetM (prhs(4))
      n = mxGetN (prhs(4))
      if (m .ne. 1 .or. n .ne. 1) then
	call mexErrMsgTxt ('nz must be a vector of length 1')
      endif
c
      m = mxGetM (prhs(5))
      n = mxGetN (prhs(5))
      if (m .ne. 1 .or. n .ne. 1) then
	call mexErrMsgTxt ('ndir must be a vector of length 1')
      endif
c
      m = mxGetM (prhs(6))
      n = mxGetN (prhs(6))
      if (max(m,n) .ne. valeur5 .or. min(m,n) .ne. 1) then
	call mexErrMsgTxt ('ixd must be a vector of length ndir')
      endif
c
      m = mxGetM (prhs(7))
      n = mxGetN (prhs(7))
      if (max(m,n) .ne. valeur5 .or. min(m,n) .ne. 1) then
	call mexErrMsgTxt ('iyd must be a vector of length ndir')
      endif
c
      m = mxGetM (prhs(8))
      n = mxGetN (prhs(8))
      if (max(m,n) .ne. valeur5 .or. min(m,n) .ne. 1) then
	call mexErrMsgTxt ('izd must be a vector of length ndir')
      endif
c
      m = mxGetM (prhs(9))
      n = mxGetN (prhs(9))
      if (m .ne. valeur2 .or. n .ne. valeur3*valeur4*valeur16) then
	call mexErrMsgTxt ('vr must be nx x ny*nz*nvar')
      endif
c      
      m = mxGetM (prhs(10))
      n = mxGetN (prhs(10))
      if (m .ne. 1 .or. n .ne. 1) then
	call mexErrMsgTxt ('tmin must be a vector of length 1')
      endif
c
      m = mxGetM (prhs(11))
      n = mxGetN (prhs(11))
      if (m .ne. 1 .or. n .ne. 1) then
	call mexErrMsgTxt ('tmax must be a vector of length 1')
      endif
c
      m = mxGetM (prhs(12))
      n = mxGetN (prhs(12))
      if (m .ne. 1 .or. n .ne. 1) then
	call mexErrMsgTxt ('nvarg must be a vector of length 1')
      endif
c
      m = mxGetM (prhs(13))
      n = mxGetN (prhs(13))
      if (max(m,n) .ne. valeur12 .or. min(m,n) .ne. 1) then
	call mexErrMsgTxt ('ivtail must be a vector of length nvarg')
      endif
c
      m = mxGetM (prhs(14))
      n = mxGetN (prhs(14))
      if (max(m,n) .ne. valeur12 .or. min(m,n) .ne. 1) then
	call mexErrMsgTxt ('ivhead must be a vector of length nvarg')
      endif
c
      m = mxGetM (prhs(15))
      n = mxGetN (prhs(15))
      if (max(m,n) .ne. valeur12 .or. min(m,n) .ne. 1) then
	call mexErrMsgTxt ('ivtype must be a vector of length nvarg')
      endif
c
      m = mxGetM (prhs(16))
      n = mxGetN (prhs(16))
      if (m .ne. 1 .or. n .ne. 1) then
	call mexErrMsgTxt ('nvar must be a vector of length 1')
      endif

c
c Create matrices for return arguments
c
      nvario  = valeur12*valeur5*valeur1
      plhs(1) = mxcreatefull (nvario,1,0)
      plhs(2) = mxcreatefull (nvario,1,0)
      plhs(3) = mxcreatefull (nvario,1,0)
      plhs(4) = mxcreatefull (nvario,1,0)
      plhs(5) = mxcreatefull (nvario,1,0)
      plhs(6) = mxcreatefull (nvario,1,0)

c
c Assign pointers to the various parameters
c
      np  = mxgetpr (plhs(1))
      gam = mxgetpr (plhs(2))
      hm  = mxgetpr (plhs(3))
      tm  = mxgetpr (plhs(4))
      hv  = mxgetpr (plhs(5))
      tv  = mxgetpr (plhs(6))

      nlag   = mxgetpr (prhs(1))
      nx     = mxgetpr (prhs(2))
      ny     = mxgetpr (prhs(3))
      nz     = mxgetpr (prhs(4))
      ndir   = mxgetpr (prhs(5))
      ixd    = mxgetpr (prhs(6))
      iyd    = mxgetpr (prhs(7))
      izd    = mxgetpr (prhs(8))
      vr     = mxgetpr (prhs(9))
      tmin   = mxgetpr (prhs(10))
      tmax   = mxgetpr (prhs(11))
      nvarg  = mxgetpr (prhs(12))
      ivtail = mxgetpr (prhs(13))
      ivhead = mxgetpr (prhs(14))
      ivtype = mxgetpr (prhs(15))
      nvar   = mxgetpr (prhs(16))

c Call the Computational Routine
c
      call gam3 (valeur1,valeur2,valeur3,valeur4,valeur5,%VAL(ixd),
     +           %VAL(iyd),%VAL(izd),%VAL(vr),%VAL(tmin),%VAL(tmax),
     +           valeur12,%VAL(ivtail),%VAL(ivhead),%VAL(ivtype),
     +           %VAL(nvar),nvario,%VAL(np),%VAL(gam),%VAL(hm),
     +           %VAL(tm),%VAL(hv),%VAL(tv))

c
c End of program
c
      return
      end
