c gamv2g.f - Gateway function for gamv2.f 
c 
c Arguments
c
c nrhs: number of right-hand side arguments
c nlhs: number of left-hand side arguments
c prhs: length nrhs array of pointers to the right-hand side matrices
c plhs: length nlhs array of pointers to the left-hand side matrices
c
c !fmex gamv2.f gamv2g.f to create Mex-File gamv2.mex
c GAMV2.MEX: Variogram of Irregularly Spaced 2-D Data
c Equivalent to GAMV2m.f without the check subroutine 
c
c Caroline Lafleur
c 30/07/96
c      

       subroutine mexFunction(nlhs, plhs, nrhs, prhs)
      
c
c Local parameters
c
      include 'gamv2.inc'     

c
c Input to the MEX-file
c
      implicit none
      integer plhs(*) , prhs(*)
      integer nlhs    , nrhs
c
      integer mxCreateFull, mxGetPr , mxCreateString
      integer mxGetM, mxGetN, mxGetString 
      real    mxGetScalar
c
c keep the above subroutine, argument, and function declarations for use
c in all your fortran mex files.
c---------------------------------------------------------------------
c

c
c GAMV2 function arguments in the syntax of the matlab language
c [np,dis,gam,hm,tm,hv,tv] = gamv2 (nd,x,y,vr,tmin,tmax,nlag,xlag,xltol,
c ndir,azm,atol,bandw,nvarg,ivtail,ivhead,ivtype,nvar)
c

c
c Pointers declaration
c
      integer nd,nlag,ndir,nvarg,nvar,ivtail,ivhead,ivtype
      integer x,y,vr,xlag,xltol,azm,atol,bandw,tmin,tmax
      integer np,dis,gam,hm,tm,hv,tv

c
c Local variables
c
      integer*4  n,m,nvario,i,xx
      integer*4  valeur1,valeur7,valeur10,valeur14,valeur18
      real*8	 valeur8,valeur9
      
c
c Check for proper number of arguments
c
      if (nrhs .ne. 18) then 
	call mexErrMsgTxt ('gamv2 requires 18 input arguments')
      else if (nlhs .ne. 7) then
	call mexErrMsgTxt ('gamv2 requires 7 output arguments')
      endif

c
c Some values needed
c
       valeur1  = int (mxGetScalar (prhs(1)))
       valeur7  = int (mxGetScalar (prhs(7)))
       valeur8  = mxGetScalar(prhs(8))
       valeur9  = mxGetScalar(prhs(9))
       valeur10 = int (mxGetScalar (prhs(10)))
       valeur14 = int (mxGetScalar (prhs(14)))
       valeur18 = int (mxgetscalar (prhs(18)))

c
c Perform some quick error checking:
c
      if(valeur10.lt.1)       stop 'ndir is too small: check parameters'
      if(valeur10.gt.maxdir)  stop 'ndir is too big: check gamv2.inc'
      if(valeur18.lt.1)       stop 'nvar is too small: check parameters'
      if(valeur18.gt.MAXVAR)  stop 'nvar is too big: check gamv2.inc'
      if(valeur7.lt.1)       stop 'nlag is too small: check parameters'
      if(valeur7.gt.maxlag)  stop 'nlag is too big: check gamv2.inc'
      if(valeur14.lt.1)      stop 'nvarg is too small: check parameters'
      if(valeur14.gt.mxvarg) stop 'nvarg is too big: check gamv2.inc'
      if(valeur8.le.0.0)  stop 'xlag is too small: check parameter file'
      if(valeur9.le.0.0) then
            write(*,*) 'xltol is too small: resetting to xlag/2'
            xltol = 0.5*xlag
      endif

c
c Check the dimension of rigth-side matrices
c
c nd,nlag,xlag,xltol,ndir,nvarg,nvar,tmin,tmax are 1x1 matrices
c azm,atol,bandw are 1xndir or ndirx1 matrices
c ivtail,ivhead,ivtype are 1xnvarg or nvargx1 matrices
c x,y,vr are vectors of length <= maxdat
c
      m = mxGetM (prhs(1))
      n = mxGetN (prhs(1))
      if (m .ne. 1 .or. n .ne. 1) then
	call mexErrMsgTxt ('nd must be a vector of length 1')
      endif
c
      m = mxGetM (prhs(2))
      n = mxGetN (prhs(2))
      if (max(m,n) .ge. maxdat .or. min(m,n) .ne. 1) then
	call mexErrMsgTxt ('The number of data exceeds the limit permited')
      endif      
      if (max(m,n) .ne. valeur1) then
        call mexErrMsgTxt ('x must be a vector of length nd')
      endif
c
      m = mxGetM (prhs(3))
      n = mxGetN (prhs(3))
      if (max(m,n) .ne. valeur1 .or. min(m,n) .ne. 1) then
	call mexErrMsgTxt ('x and y must be the same length')
      endif
c
      m = mxGetM (prhs(4))
      n = mxGetN (prhs(4))
      if (max(m,n) .ne. valeur1 .or. min(m,n) .ne. valeur18) then
	call mexErrMsgTxt ('x, y and vr must be the same length')
      endif
c
      m = mxGetM (prhs(5))
      n = mxGetN (prhs(5))
      if (m .ne. 1 .or. n .ne. 1) then
	call mexErrMsgTxt ('tmin must be a vector of length 1')
      endif
c
      m = mxGetM (prhs(6))
      n = mxGetN (prhs(6))
      if (m .ne. 1 .or. n .ne. 1) then
	call mexErrMsgTxt ('tmax must be a vector of length 1')
      endif
c
      m = mxGetM (prhs(7))
      n = mxGetN (prhs(7))
      if (m .ne. 1 .or. n .ne. 1) then
	call mexErrMsgTxt ('nlag must be a vector of length 1')
      endif
c 
      m = mxGetM (prhs(8))
      n = mxGetN (prhs(8))
      if (m .ne. 1 .or. n .ne. 1) then
	call mexErrMsgTxt ('xlag must be a vector of length 1')
      endif
c
      m = mxGetM (prhs(9))
      n = mxGetN (prhs(9))
      if (m .ne. 1 .or. n .ne. 1) then
	call mexErrMsgTxt ('xltol must be a vector of length 1')
      endif
c
      m = mxGetM (prhs(10))
      n = mxGetN (prhs(10))
      if (m .ne. 1 .or. n .ne. 1) then
	call mexErrMsgTxt ('ndir must be a vector of length 1')
      endif
c
      m = mxGetM (prhs(11))
      n = mxGetN (prhs(11))
      if (max(m,n) .ne. valeur10 .or. min(m,n) .ne. 1) then
	call mexErrMsgTxt ('azm must be a vector of length ndir')
      endif
c
      m = mxGetM (prhs(12))
      n = mxGetN (prhs(12))
      if (max(m,n) .ne. valeur10 .or. min(m,n) .ne. 1) then
	call mexErrMsgTxt ('atol must be a vector of length ndir')
      endif
c
      m = mxGetM (prhs(13))
      n = mxGetN (prhs(13))
      if (max(m,n) .ne. valeur10 .or. min(m,n) .ne. 1) then
	call mexErrMsgTxt ('bandw must be a vector of length ndir')
      endif
c
      m = mxGetM (prhs(14))
      n = mxGetN (prhs(14))
      if (m .ne. 1 .or. n .ne. 1) then
	call mexErrMsgTxt ('nvarg must be a vector of length 1')
      endif
c
      m = mxGetM (prhs(15))
      n = mxGetN (prhs(15))
      if (max(m,n) .ne. valeur14 .or. min(m,n) .ne. 1) then
	call mexErrMsgTxt ('ivtail must be a vector of length nvarg')
      endif
c
      m = mxGetM (prhs(16))
      n = mxGetN (prhs(16))
      if (max(m,n) .ne. valeur14 .or. min(m,n) .ne. 1) then
	call mexErrMsgTxt ('ivhead must be a vector of length nvarg')
      endif
c
      m = mxGetM (prhs(17))
      n = mxGetN (prhs(17))
      if (max(m,n) .ne. valeur14 .or. min(m,n) .ne. 1) then
	call mexErrMsgTxt ('ivtype must be a vector of length nvarg')
      endif
c
      m = mxgetm (prhs(18))
      n = mxgetn (prhs(18))
      if (m .ne. 1 .or. n .ne. 1) then
	call mexErrMsgTxt ('nvar must be a vector of length 1')
      endif

c
c Create matrices for return arguments
c
      nvario  = valeur14*valeur10*(valeur7+2)
      plhs(1) = mxcreatefull (nvario,1,0)
      plhs(2) = mxcreatefull (nvario,1,0)
      plhs(3) = mxcreatefull (nvario,1,0)
      plhs(4) = mxcreatefull (nvario,1,0)
      plhs(5) = mxcreatefull (nvario,1,0)
      plhs(6) = mxcreatefull (nvario,1,0)
      plhs(7) = mxcreatefull (nvario,1,0)

c
c Assign pointers to the various parameters
c
      np  = mxgetPr (plhs(1))
      dis = mxgetPr (plhs(2))
      gam = mxgetPr (plhs(3))
      hm  = mxgetPr (plhs(4))
      tm  = mxgetPr (plhs(5))
      hv  = mxgetPr (plhs(6))
      tv  = mxgetPr (plhs(7))

      nd     = mxgetpr (prhs(1))
      x      = mxgetpr (prhs(2))
      y      = mxgetpr (prhs(3))
      vr     = mxgetpr (prhs(4))
      tmin   = mxgetpr (prhs(5))
      tmax   = mxgetpr (prhs(6))
      nlag   = mxgetpr (prhs(7))
      xlag   = mxgetpr (prhs(8))
      xltol  = mxgetpr (prhs(9))
      ndir   = mxgetpr (prhs(10))
      azm    = mxgetpr (prhs(11))
      atol   = mxgetpr (prhs(12))
      bandw  = mxgetpr (prhs(13))
      nvarg  = mxgetpr (prhs(14))
      ivtail = mxgetpr (prhs(15))
      ivhead = mxgetpr (prhs(16))
      ivtype = mxgetpr (prhs(17))
      nvar   = mxgetpr (prhs(18))

c
c Call the Computational Routine
c0       1         2         3         4         5         6         7
c23456789012345678901234567890123456789012345678901234567890123456789012
c
      call gamv2 (%VAL(np),%VAL(dis),%VAL(gam),%VAL(hm),%VAL(tm),
     +		  %VAL(hv),%VAL(tv),valeur1,%VAL(x),%VAL(y),%VAL(vr),
     +            %VAL(tmin),%VAL(tmax),valeur7,%VAL(xlag),%VAL(xltol),
     +            valeur10,%VAL(azm),%VAL(atol),%VAL(bandw),valeur14,
     + 		  %VAL(ivtail),%VAL(ivhead),%VAL(ivtype),valeur18,
     +  	  nvario)

c
c End of program
c
      return
      end
