c gamv3g.f - Gateway function for gamv3.f 
c 
c Arguments
c
c nrhs: number of right-hand side arguments
c nlhs: number of left-hand side arguments
c prhs: length nrhs array of pointers to the right-hand side matrices
c plhs: length nlhs array of pointers to the left-hand side matrices
c
c !fmex gamv3.f gamv3g.f to create Mex-File gamv3.mex
c GAMV2.MEX: Variogram of Irregularly Spaced 3-D Data
c Equivalent to GAMV3m.f without the check subroutine 
c
c Caroline Lafleur
c 30/07/96
c      

       subroutine mexFunction(nlhs, plhs, nrhs, prhs)
      
c
c Local parameters
c
      include 'gamv3.inc'     

c
c Input to the MEX-file
c
      implicit none
      integer plhs(*) , prhs(*)
      integer nlhs    , nrhs
c
      integer mxCreateFull, mxGetPr , mxCreateString
      integer mxGetM, mxGetN, mxGetString 
      real    mxGetScalar
c
c keep the above subroutine, argument, and function declarations for use
c in all your fortran mex files.
c---------------------------------------------------------------------
c

c
c GAMV3 function arguments in the syntax of the matlab language
c [np,dis,gam,hm,tm,hv,tv] = gamv3 (nd,x,y,vr,tmin,tmax,nlag,xlag,xltol,
c ndir,azm,atol,bandwh,dip,dtol,bandwd,nvarg,ivtail,ivhead,ivtype,nvar)
c

c
c Pointers declaration
c
      integer nd,nlag,ndir,nvarg,nvar,ivtail,ivhead,ivtype
      integer x,y,z,vr,xlag,xltol,azm,atol,bandwh,tmin,tmax
      integer dip,dtol,bandwd
      integer np,dis,gam,hm,tm,hv,tv

c
c Local variables
c
      integer    n,m,nvario,i
      integer    valeur1,valeur8,valeur11,valeur18,valeur22
      real*8	 valeur9,valeur10

c
c Check for proper number of arguments
c
      if (nrhs .ne. 22) then 
	call mexErrMsgTxt ('gamv3 requires 22 input arguments')
      else if (nlhs .ne. 7) then
	call mexErrMsgTxt ('gamv3 requires 7 output arguments')
      endif

c
c Some values needed
c
       valeur1  = int (mxGetScalar(prhs(1)))
       valeur8  = int (mxGetScalar(prhs(8)))
       valeur9  = mxGetScalar(prhs(9))
       valeur10 = mxGetScalar(prhs(10))
       valeur11 = int(mxGetScalar (prhs(11)))
       valeur18 = int (mxGetScalar(prhs(18)))
       valeur22 = int (mxgetscalar(prhs(22)))
 
c
c Perform some quick error checking:
c      
      if(valeur11.lt.1)       stop 'ndir is too small: check parameters'
      if(valeur11.gt.MAXDIR)  stop 'ndir is too big: check gamv3.inc'
      if(valeur22.lt.1)       stop 'nvar is too small: check parameters'
      if(valeur22.gt.MAXVAR)  stop 'nvar is too big: check gamv3.inc'
      if(valeur8.lt.1)       stop 'nlag is too small: check parameters'
      if(valeur9.gt.MAXLAG)  stop 'nlag is too big: check gamv3.inc'
      if(valeur18.lt.1)      stop 'nvarg is too small: check parameters'
      if(valeur18.gt.MXVARG) stop 'nvarg is too big: check gamv3.inc'
      if(valeur9.le.0.0) stop 'xlag is too small: check parameter file'
      if(valeur10.le.0.0) then
            write(*,*) 'xltol is too small: resetting to xlag/2'
            xltol = 0.5*xlag
      endif
 
c
c Check the dimension of rigth-side matrices
c
c nd,nlag,xlag,xltol,ndir,nvarg,nvar,tmin,tmax are 1x1 matrices
c azm,atol,bandwh,dip,dtol,bandwd are 1xndir or ndirx1 matrices
c ivtail,ivhead,ivtype are 1xnvarg or nvargx1 matrices
c x,y,z,vr are vectors of length <= maxdat
c
      m = mxGetM (prhs(1))
      n = mxGetN (prhs(1))
      if (m .ne. 1 .or. n .ne. 1) then
	call mexErrMsgTxt ('nd must be a vector of length 1')
      endif
c
      m = mxGetM (prhs(2))
      n = mxGetN (prhs(2))
      if (max(m,n) .ge. maxdat .or. min(m,n) .ne. 1) then
	call mexErrMsgTxt ('The number of data exceeds the limit permited')
      endif      
      if (max(m,n) .ne. valeur1) then
        call mexErrMsgTxt ('x must be a vector of length nd')
      endif
c
      m = mxGetM (prhs(3))
      n = mxGetN (prhs(3))
      if (max(m,n) .ne. valeur1 .or. min(m,n) .ne. 1) then
	call mexErrMsgTxt ('x and y must be the same length')
      endif
c
      m = mxGetM (prhs(4))
      n = mxGetN (prhs(4))
      if (max(m,n) .ne. valeur1 .or. min(m,n) .ne. 1) then
	call mexErrMsgTxt ('x,y and z must be the same length')
      endif
c
      m = mxGetM (prhs(5))
      n = mxGetN (prhs(5))
      if (max(m,n) .ne. valeur1 .or. min(m,n) .ne. valeur22) then
	call mexErrMsgTxt ('x, y, z and vr must be the same length')
      endif
c
      m = mxGetM (prhs(6))
      n = mxGetN (prhs(6))
      if (m .ne. 1 .or. n .ne. 1) then
	call mexErrMsgTxt ('tmin must be a vector of length 1')
      endif
c
      m = mxGetM (prhs(7))
      n = mxGetN (prhs(7))
      if (m .ne. 1 .or. n .ne. 1) then
	call mexErrMsgTxt ('tmax must be a vector of length 1')
      endif
c
      m = mxGetM (prhs(8))
      n = mxGetN (prhs(8))
      if (m .ne. 1 .or. n .ne. 1) then
	call mexErrMsgTxt ('nlag must be a vector of length 1')
      endif
c 
      m = mxGetM (prhs(9))
      n = mxGetN (prhs(9))
      if (m .ne. 1 .or. n .ne. 1) then
	call mexErrMsgTxt ('xlag must be a vector of length 1')
      endif
c
      m = mxGetM (prhs(10))
      n = mxGetN (prhs(10))
      if (m .ne. 1 .or. n .ne. 1) then
	call mexErrMsgTxt ('xltol must be a vector of length 1')
      endif
c
      m = mxGetM (prhs(11))
      n = mxGetN (prhs(11))
      if (m .ne. 1 .or. n .ne. 1) then
	call mexErrMsgTxt ('ndir must be a vector of length 1')
      endif
c
      m = mxGetM (prhs(12))
      n = mxGetN (prhs(12))
      if (max(m,n) .ne. valeur11 .or. min(m,n) .ne. 1) then
	call mexErrMsgTxt ('azm must be a vector of length ndir')
      endif
c
      m = mxGetM (prhs(13))
      n = mxGetN (prhs(13))
      if (max(m,n) .ne. valeur11 .or. min(m,n) .ne. 1) then
	call mexErrMsgTxt ('atol must be a vector of length ndir')
      endif
c
      m = mxGetM (prhs(14))
      n = mxGetN (prhs(14))
      if (max(m,n) .ne. valeur11 .or. min(m,n) .ne. 1) then
	call mexErrMsgTxt ('bandwh must be a vector of length ndir')
      endif
c
      m = mxGetM (prhs(15))
      n = mxGetN (prhs(15))
      if (max(m,n) .ne. valeur11 .or. min(m,n) .ne. 1) then
	call mexErrMsgTxt ('dip must be a vector of length ndir')
      endif
c
      m = mxGetM (prhs(16))
      n = mxGetN (prhs(16))
      if (max(m,n) .ne. valeur11 .or. min(m,n) .ne. 1) then
	call mexErrMsgTxt ('dtol must be a vector of length ndir')
      endif
c
      m = mxGetM (prhs(17))
      n = mxGetN (prhs(17))
      if (max(m,n) .ne. valeur11 .or. min(m,n) .ne. 1) then
	call mexErrMsgTxt ('bandwd must be a vector of length ndir')
      endif
c
      m = mxGetM (prhs(18))
      n = mxGetN (prhs(18))
      if (m .ne. 1 .or. n .ne. 1) then
	call mexErrMsgTxt ('nvarg must be a vector of length 1')
      endif
c
      m = mxGetM (prhs(19))
      n = mxGetN (prhs(19))
      if (max(m,n) .ne. valeur18 .or. min(m,n) .ne. 1) then
	call mexErrMsgTxt ('ivtail must be a vector of length nvarg')
      endif
c
      m = mxGetM (prhs(20))
      n = mxGetN (prhs(20))
      if (max(m,n) .ne. valeur18 .or. min(m,n) .ne. 1) then
	call mexErrMsgTxt ('ivhead must be a vector of length nvarg')
      endif
c
      m = mxGetM (prhs(21))
      n = mxGetN (prhs(21))
      if (max(m,n) .ne. valeur18 .or. min(m,n) .ne. 1) then
	call mexErrMsgTxt ('ivtype must be a vector of length nvarg')
      endif
c
      m = mxgetm (prhs(22))
      n = mxgetn (prhs(22))
      if (m .ne.1 .or. n .ne. 1) then
	call mexErrMsgTxt ('nvar must be a vector of length 1')
      endif

c
c Create matrices for return arguments
c
      nvario  = valeur18*valeur11*(valeur8+2)
      plhs(1) = mxCreateFull (nvario,1,0)
      plhs(2) = mxCreateFull (nvario,1,0)
      plhs(3) = mxCreateFull (nvario,1,0)
      plhs(4) = mxCreateFull (nvario,1,0)
      plhs(5) = mxCreateFull (nvario,1,0)
      plhs(6) = mxCreateFull (nvario,1,0)
      plhs(7) = mxCreateFull (nvario,1,0)

c
c Assign pointers to the various parameters
c
      np  = mxGetPr (plhs(1))
      dis = mxGetPr (plhs(2))
      gam = mxGetPr (plhs(3))
      hm  = mxGetPr (plhs(4))
      tm  = mxGetPr (plhs(5))
      hv  = mxGetPr (plhs(6))
      tv  = mxGetPr (plhs(7))

      nd     = mxGetPr (prhs(1))
      x      = mxGetPr (prhs(2))
      y      = mxGetPr (prhs(3))
      z      = mxGetPr (prhs(4))
      vr     = mxGetPr (prhs(5))
      tmin   = mxGetPr (prhs(6))
      tmax   = mxGetPr (prhs(7))
      nlag   = mxGetPr (prhs(8))
      xlag   = mxGetPr (prhs(9))
      xltol  = mxGetPr (prhs(10))
      ndir   = mxGetPr (prhs(11))
      azm    = mxGetPr (prhs(12))
      atol   = mxGetPr (prhs(13))
      bandwh = mxGetPr (prhs(14))
      dip    = mxGetPr (prhs(15))
      dtol   = mxGetPr (prhs(16))
      bandwd = mxGetPr (prhs(17))
      nvarg  = mxGetPr (prhs(18))
      ivtail = mxGetPr (prhs(19))
      ivhead = mxGetPr (prhs(20))
      ivtype = mxGetPr (prhs(21))
      nvar   = mxgetpr (prhs(22))

c
c Call the Computational Routine
c0       1         2         3         4         5         6         7
c23456789012345678901234567890123456789012345678901234567890123456789012
c
      call gamv3 (valeur1,%VAL(x),%VAL(y),%VAL(z),%VAL(vr),%VAL(tmin),
     +   %VAL(tmax),valeur8,%VAl(xlag),%VAL(xltol),valeur11,%VAL(azm),
     +   %VAL(atol),%VAL(bandwh),%VAL(dip),%VAL(dtol),%VAL(bandwd),
     +   valeur18,%VAL(ivtail),%VAL(ivhead),%VAL(ivtype),valeur22,
     +   %VAL(np),%VAL(dis),%VAL(gam),%VAL(hm),%VAL(tm),%VAL(hv),
     +   %VAL(tv))

c
c End of program
c
      return
      end
